@@ -1,4 +1,4 @@
-/* $Id: asapinstance.c 2263 2009-10-26 08:46:55Z dreibh $
+/* $Id: asapinstance.c 2520 2011-02-19 10:05:52Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * Acknowledgements:
  * Realized in co-operation between Siemens AG and
@@ -554,7 +554,7 @@ unsigned int asapInstanceRegister(struct ASAPInstance*              asapInstance
                   result = (unsigned int)response->Error;
                }
                dispatcherUnlock(asapInstance->StateMachine);
-               
+
                if(response) {
                   rserpoolMessageDelete(response);
                }
@@ -913,7 +913,7 @@ unsigned int asapInstanceHandleResolution(
                   cacheElementTimeout);
       if(result != RSPERR_OKAY) {
          LOG_VERBOSE
-         fputs("Handle resolution not succesful\n", stdlog);
+         fputs("Handle resolution not successful\n", stdlog);
          LOG_END
       }
    }
@@ -1130,7 +1130,7 @@ static void asapInstanceHandleResponseFromRegistrar(
                  getMicroTime() - aitm->CreationTimeStamp,
                  aitm->TransmissionTimeStamp - aitm->CreationTimeStamp);
          LOG_END
-         
+
          /* Asynchronous message: print errors here! */
          if(aitm->Node.ReplyPort == NULL) {
             if( (response->Type == AHT_REGISTRATION_RESPONSE) &&
@@ -1143,13 +1143,13 @@ static void asapInstanceHandleResponseFromRegistrar(
                rserpoolErrorPrint(response->Error, stdlog);
                fputs("\n", stdlog);
                LOG_END
-               
+
                /* If no addresses are usable, try to re-connect. May be, the
                   host has got some usable addresses then. */
                if( (response->Error == RSPERR_NO_USABLE_USER_ADDRESSES) ||
                    (response->Error == RSPERR_NO_USABLE_ASAP_ADDRESSES) ) {
                   LOG_NOTE
-                  fputs("Trying to re-connect to a registrar ...\n", stdlog);   
+                  fputs("Trying to re-connect to a registrar ...\n", stdlog);
                   LOG_END
                   asapInstanceDisconnectFromRegistrar(asapInstance, true);
                }
@@ -1470,7 +1470,7 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
                        aitm->ResponseTimeoutTimeStamp);
          }
       }
-      aitm->ResponseTimeoutNeedsScheduling = false;         
+      aitm->ResponseTimeoutNeedsScheduling = false;
       aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortGetNextMessage(&asapInstance->MainLoopPort, &aitm->Node);
    }
    interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
@@ -1481,7 +1481,12 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
 /* ###### Wake up main loop thread using pipe ############################ */
 static void asapInstanceNotifyMainLoop(struct ASAPInstance* asapInstance)
 {
-   ext_write(asapInstance->MainLoopPipe[1], "!", 1);
+   const ssize_t result = ext_write(asapInstance->MainLoopPipe[1], "!", 1);
+   if(result <= 0) {
+      LOG_ERROR
+      logerror("Writing to main loop pipe failed");
+      LOG_END
+   }
 }
 
 
@@ -1506,6 +1511,7 @@ static void* asapInstanceMainLoop(void* args)
    int                  timeout;
    unsigned int         pipeIndex;
    int                  result;
+   ssize_t              r;
 
    asapInstanceConnectToRegistrar(asapInstance, -1);
 
@@ -1533,7 +1539,12 @@ static void* asapInstanceMainLoop(void* args)
       dispatcherHandlePollResult(asapInstance->StateMachine, result,
                                  (struct pollfd*)&ufds, nfds, timeout, pollTimeStamp);
       if(ufds[pipeIndex].revents & POLLIN) {
-         CHECK(ext_read(asapInstance->MainLoopPipe[0], (char*)&buffer, sizeof(buffer)) > 0);
+         r = ext_read(asapInstance->MainLoopPipe[0], (char*)&buffer, sizeof(buffer));
+         if(r <= 0) {
+            LOG_ERROR
+            logerror("Reading from main loop pipe failed");
+            LOG_END
+         }
       }
 
       /* ====== Handle inter-thread messages ============================= */
