@@ -1,4 +1,4 @@
-/* $Id: fractalpooluser.cc 2272 2009-11-15 11:36:26Z dreibh $
+/* $Id: fractalpooluser.cc 2460 2011-02-01 07:21:55Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -341,7 +341,7 @@ void FractalPU::startNextJob()
    // ====== Wait for job completion ========================================
    // std::cout << "Waiting for job completion ..." << std::endl;
    if(CurrentThreads > 1) {
-      statusBar()->showMessage("Waiting for job completion ...");
+      statusBar()->showMessage(tr("Waiting for job completion ..."));
    }
 }
 
@@ -447,7 +447,7 @@ void FractalPU::contextMenuEvent(QContextMenuEvent* event)
    showSessions->setCheckable(true);
    showSessions->setChecked(ShowSessions);
    connect(showSessions, SIGNAL(toggled(bool)), this, SLOT(changeShowSessions(bool)));
-   
+
    QMenu* sessionsMenu = menu.addMenu(tr("Sessions"));
    Q_CHECK_PTR(sessionsMenu);
    for(size_t i = 0;i < sizeof(sessionsEntries) / sizeof(sessionsEntries[0]);i++) {
@@ -467,6 +467,7 @@ void FractalPU::contextMenuEvent(QContextMenuEvent* event)
    connect(sessionsMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeThreads(QAction*)));
 
    menu.addSeparator();
+   menu.addAction(tr("&Next Image"), this, SLOT(restart()));
    menu.addAction(tr("&About"), this, SLOT(about()));
    menu.addSeparator();
    menu.addAction(tr("&Quit"), this, SLOT(quit()));
@@ -500,13 +501,20 @@ void FractalPU::changeThreads(QAction* action)
 }
 
 
+/* ###### Next Image ##################################################### */
+void FractalPU::restart()
+{
+   setStatus(FPU_CalcAborted);
+   CountDown = 0;
+}
+
 /* ###### About ########################################################## */
 void FractalPU::about()
 {
    QMessageBox::about(this, tr("About Fractal Pool User"),
       "<center>"
          "<b>Fractal Pool User</b><br>"
-         "Copyright (C) 2002-2009 by Thomas Dreibholz"
+         "Copyright (C) 2002-2011 by Thomas Dreibholz"
       "</center>");
 }
 
@@ -654,22 +662,22 @@ void FractalCalculationThread::run()
                do {
 
                   // ====== Handle received result chunks ===================
-                  size_t         packets = 0;
-                  FGPData        data;
                   rsp_sndrcvinfo rinfo;
-                  int            flags = 0;
-                  ssize_t received;
+                  char           buffer[max(sizeof(FGPData), sizeof(rsp_sndrcvinfo))];
+                  int            flags   = 0;
+                  size_t         packets = 0;
+                  ssize_t        received;
 
-                  received = rsp_recvfullmsg(Session, (char*)&data, sizeof(data),
+                  received = rsp_recvfullmsg(Session, (char*)&buffer, sizeof(buffer),
                                              &rinfo, &flags, Master->getRecvTimeout());
                   while(received > 0) {
                      // ====== Handle notification ==========================
                      if(flags & MSG_RSERPOOL_NOTIFICATION) {
-                        union rserpool_notification* notification =
-                           (union rserpool_notification*)&data;
+                        const union rserpool_notification* notification =
+                           (const union rserpool_notification*)&buffer;
                         printTimeStamp(stdout);
                         printf("Notification: ");
-                        rsp_print_notification((union rserpool_notification*)&data, stdout);
+                        rsp_print_notification(notification, stdout);
                         puts("");
                         if(notification->rn_header.rn_type == RSERPOOL_FAILOVER) {
                            if(notification->rn_failover.rf_state == RSERPOOL_FAILOVER_NECESSARY) {
@@ -683,9 +691,10 @@ void FractalCalculationThread::run()
 
                      // ====== Handle Data ==================================
                      else {
+                        const FGPData* data = (const FGPData*)&buffer;
                         if((received >= (ssize_t)sizeof(FGPCommonHeader)) &&
-                           (data.Header.Type == FGPT_DATA)) {
-                           const DataStatus status = handleDataMessage(&data, received);
+                           (data->Header.Type == FGPT_DATA)) {
+                           const DataStatus status = handleDataMessage(data, received);
 
                            switch(status) {
                               case Finalizer:
@@ -718,7 +727,7 @@ void FractalCalculationThread::run()
                      }
 
                      flags = 0;
-                     received = rsp_recvfullmsg(Session, (char*)&data, sizeof(data),
+                     received = rsp_recvfullmsg(Session, (char*)&buffer, sizeof(buffer),
                                                 &rinfo, &flags, Master->getRecvTimeout());
                   }
                   if(Success == false) {
@@ -953,7 +962,7 @@ int main(int argc, char** argv)
                     "2.0kde",
                     ki18n("A KDE/RSerPool Fractal Generator Pool User"),
                     KAboutData::License_GPL_V3,
-                    ki18n("Copyright (C) 2009 Thomas Dreibholz"));
+                    ki18n("Copyright (C) 2009-2011 Thomas Dreibholz"));
    KCmdLineArgs::init(argc, argv, &about);
    KApplication application;
 #else
