@@ -1,4 +1,4 @@
-/* $Id: netutilities.c 2259 2009-10-25 14:56:13Z dreibh $
+/* $Id: netutilities.c 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * Acknowledgements:
  * Realized in co-operation between Siemens AG and
@@ -58,9 +58,13 @@
 #include <assert.h>
 #ifdef SOLARIS
 #include <sys/sockio.h>
+#ifndef CMSG_SPACE
 #define CMSG_SPACE(len) (_CMSG_HDR_ALIGN(sizeof(struct cmsghdr)) + _CMSG_DATA_ALIGN(len))
+#endif
+#ifndef CMSG_LEN
 #define CMSG_LEN(len) (_CMSG_HDR_ALIGN(sizeof(struct cmsghdr)) + (len))
 #endif
+#endif
 #ifdef HAVE_VALGRIND_MEMCHECK_H
 #include <valgrind/memcheck.h>
 #endif
@@ -635,7 +639,7 @@ size_t filterAddressesByScope(union sockaddr_union* addressArray,
 {
    size_t outputAddresses = 0;
    size_t i;
-   
+
    for(i = 0;i < inputAddresses;i++) {
       if(getScope(&addressArray[i].sa) >= minScope) {
          if(outputAddresses != i) {
@@ -1409,6 +1413,16 @@ int sendtoplus(int                      sockfd,
                           (struct sockaddr*)toaddrs,
                           (toaddrs != NULL) ? getSocklen((struct sockaddr*)toaddrs) : 0);
    }
+#ifdef LINUX
+#warning Using lksctp SCTP_EOF/SCTP_ABORT shutdown work-around for TCP-like sockets!
+   /* LK-SCTP refuses SCTP_EOF and SCTP_ABORT on TCP-like socket.
+      => using ext_shutdown() instead! */
+   if( (result < 0) &&
+       (assocID == 0) &&   /* TCP-like socket */
+       ((sctpFlags & SCTP_EOF) || (sctpFlags & SCTP_ABORT)) ) {
+      ext_shutdown(sockfd, 2);
+   }
+#endif
 
    if((timeout > 0) && ((result < 0) && (errno == EWOULDBLOCK))) {
       remainingTimeout = timeout;
@@ -1975,7 +1989,10 @@ size_t getladdrsplus(const int              fd,
          ( ( (((struct sockaddr*)packedAddresses)->sa_family == AF_INET) &&
                 (((struct sockaddr_in*)packedAddresses)->sin_addr.s_addr == 0) ) ||
              ( (((struct sockaddr*)packedAddresses)->sa_family == AF_INET6) &&
-                (IN6_IS_ADDR_UNSPECIFIED(&((struct sockaddr_in6*)packedAddresses)->sin6_addr))))) {
+                ( (IN6_IS_ADDR_UNSPECIFIED(&((struct sockaddr_in6*)packedAddresses)->sin6_addr)) ||
+                  ( ((IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6*)packedAddresses)->sin6_addr) ||
+                     (IN6_IS_ADDR_V4COMPAT(&((struct sockaddr_in6*)packedAddresses)->sin6_addr))) &&
+                    (((struct sockaddr_in6*)packedAddresses)->sin6_addr.s6_addr32[3] == 0))))))) {
          port = getPort((struct sockaddr*)packedAddresses);
          sctp_freeladdrs(packedAddresses);
 
@@ -2051,21 +2068,35 @@ size_t getpaddrsplus(const int              fd,
 /* ###### Send SCTP ABORT ################################################ */
 int sendabort(int sockfd, sctp_assoc_t assocID)
 {
+#ifdef SOLARIS
+   /* The Solaris API has no SCTP_ABORT. Instead, it is a regular flag. */
+   return(sendtoplus(sockfd, NULL, 0,
+                     MSG_ABORT,
+                     NULL, 0,
+                     0, assocID, 0, 0, 0, 0));
+#else
    return(sendtoplus(sockfd, NULL, 0,
 #ifdef MSG_NOSIGNAL
                      MSG_NOSIGNAL,
 #else
                      0,
 #endif
-
                      NULL, 0,
                      0, assocID, 0, 0, SCTP_ABORT, 0));
+#endif
 }
 
 
 /* ###### Send SCTP SHUTDOWN ############################################# */
 int sendshutdown(int sockfd, sctp_assoc_t assocID)
 {
+#ifdef SOLARIS
+   /* The Solaris API has no SCTP_ABORT. Instead, it is a regular flag. */
+   return(sendtoplus(sockfd, NULL, 0,
+                     MSG_EOF,
+                     NULL, 0,
+                     0, assocID, 0, 0, 0, 0));
+#else
    return(sendtoplus(sockfd, NULL, 0,
 #ifdef MSG_NOSIGNAL
                      MSG_NOSIGNAL,
@@ -2074,6 +2105,7 @@ int sendshutdown(int sockfd, sctp_assoc_t assocID)
 #endif
                      NULL, 0,
                      0, assocID, 0, 0, SCTP_EOF, 0));
+#endif
 }
 
 
@@ -2107,3 +2139,113 @@ uint64_t ntoh64(const uint64_t value)
 {
    return(byteswap64(value));
 }
+
+
+/* ###### Tune SCTP parameters ############################################ */
+bool tuneSCTP(const int                  sockfd,
+              sctp_assoc_t               assocID,
+              struct TuneSCTPParameters* parameters)
+{
+   struct sctp_rtoinfo     rtoinfo;
+   struct sctp_paddrparams peerParams;
+   struct sctp_assocparams assocParams;
+   socklen_t               size;
+   bool                    result = true;
+
+   LOG_VERBOSE3
+   fprintf(stdlog, "Tuning SCTP parameters of assoc %u:\n", sockfd);
+   LOG_END
+
+   size = (socklen_t)sizeof(rtoinfo);
+   rtoinfo.srto_assoc_id = assocID;
+   if(ext_getsockopt(sockfd, IPPROTO_SCTP, SCTP_RTOINFO, &rtoinfo, &size) < 0) {
+      LOG_WARNING
+      logerror("getsockopt SCTP_RTOINFO failed -> skipping RTO configuration");
+      LOG_END
+      return(false);
+   }
+   size = (socklen_t)sizeof(peerParams);
+   peerParams.spp_assoc_id = assocID;
+   memset(&peerParams.spp_address, 0, sizeof(peerParams.spp_address));   // ANY address
+   if(ext_getsockopt(sockfd, IPPROTO_SCTP, SCTP_PEER_ADDR_PARAMS, &peerParams, &size) < 0) {
+      LOG_WARNING
+      logerror("getsockopt SCTP_PEER_ADDR_PARAMS failed -> skipping path configuration");
+      LOG_END
+      return(false);
+   }
+   size = (socklen_t)sizeof(assocParams);
+   assocParams.sasoc_assoc_id = assocID;
+   if(ext_getsockopt(sockfd, IPPROTO_SCTP, SCTP_ASSOCINFO, &assocParams, &size) < 0) {
+      LOG_WARNING
+      logerror("getsockopt SCTP_ASSOCINFO failed -> skipping assoc configuration");
+      LOG_END
+      return(false);
+   }
+
+   LOG_VERBOSE3
+   fputs("Old configuration:\n", stdlog);
+   fprintf(stdlog, " - Initial RTO:       %u ms\n", rtoinfo.srto_initial);
+   fprintf(stdlog, " - Min RTO:           %u ms\n", rtoinfo.srto_min);
+   fprintf(stdlog, " - Max RTO:           %u ms\n", rtoinfo.srto_max);
+   fprintf(stdlog, " - HeartbeatInterval: %u ms\n", peerParams.spp_hbinterval);
+   fprintf(stdlog, " - PathMaxRXT:        %u\n",    peerParams.spp_pathmaxrxt);
+   fprintf(stdlog, " - AssocMaxRXT:       %u\n",    assocParams.sasoc_asocmaxrxt);
+   LOG_END
+
+   if(parameters->InitialRTO != 0) {
+      rtoinfo.srto_initial = parameters->InitialRTO;
+   }
+   if(parameters->MinRTO != 0) {
+      rtoinfo.srto_min = parameters->MinRTO;
+   }
+   if(parameters->MaxRTO != 0) {
+      rtoinfo.srto_max = parameters->MaxRTO;
+   }
+   peerParams.spp_hbinterval = parameters->HeartbeatInterval;
+   peerParams.spp_flags      = 0;
+   if(parameters->HeartbeatInterval != 0) {
+      peerParams.spp_flags |= SPP_HB_ENABLE;
+   }
+   if(parameters->PathMaxRxt != 0) {
+      peerParams.spp_pathmaxrxt = parameters->PathMaxRxt;
+   }
+   if(parameters->AssocMaxRxt != 0) {
+      assocParams.sasoc_asocmaxrxt = parameters->AssocMaxRxt;
+   }
+
+   if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_ASSOCINFO, &assocParams, (socklen_t)sizeof(assocParams)) < 0) {
+      LOG_WARNING
+      logerror("setsockopt SCTP_ASSOCINFO failed");
+      LOG_END
+      result = false;
+   }
+
+   if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_RTOINFO, &rtoinfo, (socklen_t)sizeof(rtoinfo)) < 0) {
+      LOG_WARNING
+      logerror("setsockopt SCTP_RTOINFO failed");
+      LOG_END
+      result = false;
+   }
+   peerParams.spp_assoc_id = assocID;
+   memset(&peerParams.spp_address, 0, sizeof(peerParams.spp_address));   // ANY address
+   if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_PEER_ADDR_PARAMS, &peerParams, sizeof(peerParams)) < 0) {
+      LOG_WARNING
+      logerror("setsockopt SCTP_PEER_ADDR_PARAMS failed");
+      LOG_END
+      result = false;
+   }
+
+   if(result) {
+      LOG_VERBOSE3
+      fputs("New configuration:\n", stdlog);
+      fprintf(stdlog, " - Initial RTO:       %u ms\n", rtoinfo.srto_initial);
+      fprintf(stdlog, " - Min RTO:           %u ms\n", rtoinfo.srto_min);
+      fprintf(stdlog, " - Max RTO:           %u ms\n", rtoinfo.srto_max);
+      fprintf(stdlog, " - HeartbeatInterval: %u ms\n", peerParams.spp_hbinterval);
+      fprintf(stdlog, " - PathMaxRXT:        %u\n",    peerParams.spp_pathmaxrxt);
+      fprintf(stdlog, " - AssocMaxRXT:       %u\n",    assocParams.sasoc_asocmaxrxt);
+      LOG_END
+   }
+
+   return(result);
+}
