@@ -1,4 +1,4 @@
-/* $Id: rserpool.h 2207 2009-09-11 09:48:04Z dreibh $
+/* $Id: rserpool.h 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,7 +91,7 @@ struct rsp_info
    unsigned int               ri_registrar_request_timeout;
    unsigned int               ri_registrar_response_timeout;
    unsigned int               ri_registrar_request_max_trials;
-   
+
    uint64_t                   ri_csp_identifier;
    struct sockaddr*           ri_csp_server;
    unsigned int               ri_csp_interval;
@@ -202,7 +202,7 @@ unsigned int rsp_pe_failure(const unsigned char* poolHandle,
   * @param rserpoolAddrInfo Pointer to variable to store pointer to first rsp_addrinfo to.
   * @param items Desired number of PE entries to obtain.
   * @param staleCacheValue Stale cache value in milliseconds.
-  * @return Number of PE entries obtained in case of success; -1 in case of an error.
+  * @return Number of PE entries obtained in case of success; error code (negative) in case of an error.
   *
   * @see rsp_freeaddrinfo
   */
@@ -212,6 +212,12 @@ int rsp_getaddrinfo(const unsigned char*  poolHandle,
                     const size_t          items,
                     const unsigned int    staleCacheValue);
 
+/* Error values for rsp_getaddrinfo() function. */
+#define REAI_NONAME -1   /* Pool Handle is unknown.           */
+#define REAI_MEMORY -2   /* Memory allocation failure.        */
+#define REAI_SYSTEM -3   /* System error returned in `errno'. */
+#define REAI_FAMILY -4   /* Address family not supported.     */
+
 /**
   * Free rsp_addrinfo structure.
   *
@@ -300,6 +306,17 @@ union rserpool_notification
 #define RSERPOOL_SHUTDOWN_EVENT 3
 
 
+struct rserpool_session_parameters
+{
+   unsigned int sp_rto_initial;
+   unsigned int sp_rto_min;
+   unsigned int sp_rto_max;
+   unsigned int sp_assoc_max_rxt;
+   unsigned int sp_path_max_rxt;
+   unsigned int sp_hbinterval;
+};
+
+
 /* #######################################################################
    #### RSerPool Socket Functions                                     ####
    ####################################################################### */
@@ -314,6 +331,18 @@ union rserpool_notification
   */
 int rsp_socket(int domain, int type, int protocol);
 
+/**
+  * Update session parameters with given rserpool_session_parameters structure.
+  * Values set to 0 remain unchanged. The values actually set are returned in
+  * the structure.
+  *
+  * @param sd Socket descriptor.
+  * @param params rserpool_session_parameters structure.
+  * @return 0 in case of success; -1 in case of an error.
+  */
+int rsp_update_session_parameters(int sd,
+                                  struct rserpool_session_parameters* params);
+
 /**
   * Bind RSerPool socket to address(es).
   *
@@ -359,6 +388,7 @@ int rsp_getpeername(int            sd,
                     size_t*        poolHandleSize,
                     uint32_t*      identifier);
 
+#if 0
 /**
   *
   *
@@ -367,6 +397,7 @@ int rsp_getpeername(int            sd,
   * @return 0 in case of success; -1 in case of an error.
   */
 int rsp_();
+#endif
 
 /**
   * Close RSerPool socket.
