@@ -1,4 +1,4 @@
-/* $Id: rserpoolsocket.c 2190 2009-08-05 07:43:39Z dreibh $
+/* $Id: rserpoolsocket.c 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -172,6 +172,7 @@ bool doRegistration(struct RSerPoolSocket* rserpoolSocket,
    struct rsp_addrinfo*  rspAddrInfo;
    struct rsp_addrinfo*  rspAddrInfo2;
    struct rsp_addrinfo*  next;
+   size_t                oldSctpLocalAddresses;
    size_t                sctpLocalAddresses;
    union sockaddr_union* sctpLocalAddressArray = NULL;
    union sockaddr_union* localAddressArray     = NULL;
@@ -225,11 +226,17 @@ bool doRegistration(struct RSerPoolSocket* rserpoolSocket,
          /* --------------------------------------------- */
 
          /* ====== Filter out link-local addresses ======================= */
+         oldSctpLocalAddresses = sctpLocalAddresses;
          sctpLocalAddresses = filterAddressesByScope(sctpLocalAddressArray, sctpLocalAddresses,
                                                      AS_UNICAST_SITELOCAL);
          if(sctpLocalAddresses == 0) {
             LOG_ERROR
-            fputs("No addresses of at least site-local scope found -> No registration possible\n", stdlog);
+            fputs("No addresses of at least site-local scope found -> No registration possible. Addresses:\n", stdlog);
+            for(i = 0; i < oldSctpLocalAddresses;i++) {
+               fputs(" - ", stdlog);
+               fputaddress(&sctpLocalAddressArray[i].sa, true, stdlog);
+               fputs("\n", stdlog);
+            }
             LOG_END
             return(false);
          }
