@@ -1,4 +1,4 @@
-/* $Id: scriptingclient.c 2263 2009-10-26 08:46:55Z dreibh $
+/* $Id: scriptingclient.c 2522 2011-02-19 10:07:13Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "netutilities.h"
 #include "randomizer.h"
 #include "netutilities.h"
+#include "sha1.h"
 #ifdef ENABLE_CSP
 #include "componentstatuspackets.h"
 #endif
@@ -51,12 +52,15 @@
 #define SSCR_FAILOVER    2
 
 #define SSCS_WAIT_SERVER_READY     1
-#define SSCS_WAIT_START_PROCESSING 2
-#define SSCS_PROCESSING            3
-#define SSCS_DOWNLOAD              4
+#define SSCS_WAIT_ENVIRONMENT      2
+#define SSCS_WAIT_START_PROCESSING 3
+#define SSCS_PROCESSING            4
+#define SSCS_DOWNLOAD              5
 
 
-static unsigned int       State             = SSCS_WAIT_SERVER_READY;
+static uint8_t            EnvironmentHash[SE_HASH_SIZE];
+static unsigned long long EnvironmentSize   = 0;
+static const char*        EnvironmentName   = NULL;
 static const char*        InputName         = NULL;
 static const char*        OutputName        = NULL;
 static FILE*              OutputFile        = NULL;
@@ -67,6 +71,7 @@ static unsigned long long KeepAliveInterval = 10000000;
 static unsigned long long KeepAliveTimeout  = 10000000;
 static unsigned long long RetryDelay        = 10000000;
 static const char*        RunID             = NULL;
+static unsigned int       State             = SSCS_WAIT_SERVER_READY;
 static unsigned int       Trial             = 1;
 static uint32_t           ExitStatus        = 0;
 static bool               HasAssignedPE     = false;
@@ -102,16 +107,16 @@ static void newLogLine(FILE* fh)
 
 
 /* ###### Upload input file ############################################## */
-static unsigned int performUpload(int sd)
+static unsigned int performUpload(int sd, const char* name)
 {
    struct Upload upload;
    size_t        dataLength;
    ssize_t       sent;
 
-   FILE* fh = fopen(InputName, "r");
+   FILE* fh = fopen(name, "r");
    if(fh == NULL) {
       printTimeStamp(stderr);
-      fprintf(stderr, "ERROR: Unable to open input file \"%s\"!\n", InputName);
+      fprintf(stderr, "ERROR: Unable to open file \"%s\"!\n", name);
       exit(1);
    }
 
@@ -132,14 +137,14 @@ static unsigned int performUpload(int sd)
             fflush(stdout);
             return(SSCR_FAILOVER);
          }
-         
+
          if(dataLength == 0) {
             break;
          }
       }
       else {
          newLogLine(stderr);
-         fprintf(stderr, "ERROR: Reading failed in \"%s\"!\n", InputName);
+         fprintf(stderr, "ERROR: Reading failed in \"%s\"!\n", name);
          exit(1);
       }
    }
@@ -153,8 +158,8 @@ static unsigned int performUpload(int sd)
 
 
 /* ###### Handle Status message ############################################# */
-static unsigned int handleStatus(const struct Status* status,
-                                 const size_t         length)
+static unsigned int handleStatusMessage(const struct Status* status,
+                                        const size_t         length)
 {
    if(length >= sizeof(struct Status)) {
       ExitStatus = ntohl(status->Status);
@@ -172,7 +177,7 @@ static unsigned int handleStatus(const struct Status* status,
          newLogLine(stderr);
          fputs("Maximum number of trials reached -> check your input and the results!\n", stderr);
          fputs("Trying to download the results file for debugging ...", stderr);
-         return(SSCR_COMPLETE);
+         /* Final trial has failed: continue with download of the error output! */
       }
       return(SSCR_OKAY);
    }
@@ -186,7 +191,7 @@ static unsigned int handleStatus(const struct Status* status,
 
 
 /* ###### Send KeepAlive message ######################################### */
-static unsigned int sendKeepAlive(int sd)
+static unsigned int sendKeepAliveMessage(int sd)
 {
    struct KeepAlive keepAlive;
    ssize_t          sent;
@@ -198,6 +203,7 @@ static unsigned int sendKeepAlive(int sd)
                       0, htonl(PPID_SP), 0, 0, 0, 0);
    if(sent != sizeof(keepAlive)) {
       if((State != SSCS_WAIT_SERVER_READY) &&
+         (State != SSCS_WAIT_ENVIRONMENT) &&
          (State != SSCS_WAIT_START_PROCESSING)) {
          newLogLine(stdout);
          printf("Keep-Alive transmission error in state #%u: %s\n", State, strerror(errno));
@@ -217,7 +223,7 @@ static unsigned int sendKeepAlive(int sd)
 
 
 /* ###### Handle KeepAliveAck message #################################### */
-static unsigned int handleKeepAliveAck()
+static unsigned int handleKeepAliveAckMessage()
 {
    KeepAliveTransmitted = false;
    LastKeepAlive        = getMicroTime();
@@ -226,7 +232,7 @@ static unsigned int handleKeepAliveAck()
 
 
 /* ###### Send KeepAliveAck message ###################################### */
-static unsigned int sendKeepAliveAck(int sd)
+static unsigned int sendKeepAliveAckMessage(int sd)
 {
    struct KeepAlive keepAliveAck;
    ssize_t          sent;
@@ -241,9 +247,9 @@ static unsigned int sendKeepAliveAck(int sd)
 
 
 /* ###### Handle Ready message ########################################### */
-static void handleNotReady(const int              sd,
-                           const struct NotReady* notReady,
-                           const size_t           length)
+static void handleNotReadyMessage(const int              sd,
+                                  const struct NotReady* notReady,
+                                  const size_t           length)
 {
    if(length >= sizeof(struct NotReady)) {
       /* ====== Get info string ========================================== */
@@ -266,9 +272,9 @@ static void handleNotReady(const int              sd,
 
 
 /* ###### Handle Ready message ########################################### */
-static unsigned int handleReady(const int           sd,
-                                const struct Ready* ready,
-                                const size_t        length)
+static unsigned int handleReadyMessage(const int           sd,
+                                       const struct Ready* ready,
+                                       const size_t        length)
 {
    if(length < sizeof(struct Ready)) {
       newLogLine(stdout);
@@ -276,7 +282,7 @@ static unsigned int handleReady(const int           sd,
       fflush(stdout);
       return(SSCR_FAILOVER);
    }
-   
+
    /* ====== Get info string ============================================= */
    size_t i;
    for(i = 0;i < (length - sizeof(struct Ready));i++) {
@@ -289,14 +295,70 @@ static unsigned int handleReady(const int           sd,
    HasAssignedPE = true;
 
    /* ====== Proceed with uploading the work package ===================== */
-   State = SSCS_WAIT_START_PROCESSING;
    newLogLine(stdout);
    printf("Server %s is ready!\n", InfoString);
    fflush(stdout);
-   return(performUpload(sd));
+
+   /* ====== Request environment ========================================= */
+   if(EnvironmentName != NULL) {
+      State = SSCS_WAIT_ENVIRONMENT;
+      struct Environment environment;
+      environment.Header.Type   = SPT_ENVIRONMENT;
+      environment.Header.Flags  = 0x00;
+      environment.Header.Length = htons(sizeof(environment));
+      memcpy(&environment.Hash, EnvironmentHash, sizeof(EnvironmentHash));
+      ssize_t sent = rsp_sendmsg(sd, (const char*)&environment, sizeof(environment), 0,
+                                 0, htonl(PPID_SP), 0, 0, 0, (int)(TransmitTimeout / 1000));
+      if(sent != (ssize_t)sizeof(environment)) {
+         newLogLine(stdout);
+         printf("Environment query error: %s\n", strerror(errno));
+         fflush(stdout);
+         return(SSCR_FAILOVER);
+      }
+   }
+   else {
+      State = SSCS_WAIT_START_PROCESSING;
+   }
+
+   /* ====== Continue with input upload ================================== */
+   return(performUpload(sd, InputName));
 }
 
 
+/* ###### Handle Ready message ########################################### */
+static unsigned int handleEnvironmentMessage(const int                 sd,
+                                             const struct Environment* environment,
+                                             const size_t              length)
+{
+   if(length < sizeof(struct Environment)) {
+      newLogLine(stdout);
+      puts("Received invalid Environment message!");
+      fflush(stdout);
+      return(SSCR_FAILOVER);
+   }
+
+   /* ====== Proceed with uploading the work package ===================== */
+   State = SSCS_WAIT_START_PROCESSING;
+   newLogLine(stdout);
+   if(environment->Header.Flags & SEF_UPLOAD_NEEDED) {
+      if(EnvironmentName != NULL) {
+         printf("Server %s needs environment\n", InfoString);
+         fflush(stdout);
+         return(performUpload(sd, EnvironmentName));
+      }
+      else {
+         printf("Server %s has requested environment, but there is no environment?!\n", InfoString);
+         fflush(stdout);
+         return(SSCR_FAILOVER);
+      }
+   }
+   else {
+      printf("Server %s has cached environment -> skipping upload\n", InfoString);
+      fflush(stdout);
+      return(SSCR_OKAY);
+   }
+}
+
 /* ###### Handle Status message ########################################## */
 static unsigned int serverStartsProcessing(const int            sd,
                                            const struct Status* status,
@@ -308,7 +370,7 @@ static unsigned int serverStartsProcessing(const int            sd,
       fflush(stdout);
       return(SSCR_FAILOVER);
    }
-   
+
    if(ntohl(status->Status) == 0) {
       State = SSCS_PROCESSING;
       newLogLine(stdout);
@@ -326,8 +388,8 @@ static unsigned int serverStartsProcessing(const int            sd,
 
 
 /* ###### Handle Download message ######################################## */
-static unsigned int handleDownload(const struct Download* download,
-                                   const size_t           length)
+static unsigned int handleDownloadMessage(const struct Download* download,
+                                          const size_t           length)
 {
    size_t dataLength;
 
@@ -375,13 +437,17 @@ static unsigned int handleMessage(int                                 sd,
                                   const size_t                        length,
                                   const uint32_t                      ppid)
 {
+   /* printf("Message %u in state %u ...\n", header->Type, State); */
+
    /* ====== Check message header ======================================== */
+/*
    if(ppid != PPID_SP) {
       newLogLine(stdout);
       printf("Received message has wrong PPID $%08x!\n", ppid);
       fflush(stdout);
       return(SSCR_FAILOVER);
    }
+*/
    if(length < sizeof(struct ScriptingCommonHeader)) {
       newLogLine(stdout);
       printf("Received message of %u bytes does not even contain header!\n",
@@ -404,15 +470,24 @@ static unsigned int handleMessage(int                                 sd,
       case SSCS_WAIT_SERVER_READY:
          switch(header->Type) {
             case SPT_READY:
-               return(handleReady(sd, (const struct Ready*)header, length));
+               return(handleReadyMessage(sd, (const struct Ready*)header, length));
              break;
             case SPT_NOTREADY:
-               handleNotReady(sd, (const struct NotReady*)header, length);
+               handleNotReadyMessage(sd, (const struct NotReady*)header, length);
                return(SSCR_FAILOVER);
              break;
          }
         break;
 
+      /* ====== Wait for Ready message to start upload =================== */
+      case SSCS_WAIT_ENVIRONMENT:
+         switch(header->Type) {
+            case SPT_ENVIRONMENT:
+               return(handleEnvironmentMessage(sd, (const struct Environment*)header, length));
+             break;
+         }
+        break;
+
       /* ====== Wait for Ready message to start upload =================== */
       case SSCS_WAIT_START_PROCESSING:
          switch(header->Type) {
@@ -421,19 +496,19 @@ static unsigned int handleMessage(int                                 sd,
              break;
          }
         break;
-        
+
       /* ====== Processing =============================================== */
       case SSCS_PROCESSING:
          switch(header->Type) {
             case SPT_KEEPALIVE_ACK:
-             return(handleKeepAliveAck());
+             return(handleKeepAliveAckMessage());
              break;
             case SPT_KEEPALIVE:
-               return(sendKeepAliveAck(sd));
+               return(sendKeepAliveAckMessage(sd));
              break;
             case SPT_STATUS:
                State = SSCS_DOWNLOAD;
-               return(handleStatus((const struct Status*)header, length));
+               return(handleStatusMessage((const struct Status*)header, length));
              break;
          }
        break;
@@ -442,10 +517,10 @@ static unsigned int handleMessage(int                                 sd,
       case SSCS_DOWNLOAD:
          switch(header->Type) {
             case SPT_DOWNLOAD:
-               return(handleDownload((const struct Download*)header, length));
+               return(handleDownloadMessage((const struct Download*)header, length));
              break;
             case SPT_KEEPALIVE_ACK:
-               return(handleKeepAliveAck());
+               return(handleKeepAliveAckMessage());
              break;
          }
        break;
@@ -461,7 +536,6 @@ static unsigned int handleMessage(int                                 sd,
 }
 
 
-
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
 {
@@ -488,6 +562,12 @@ int main(int argc, char** argv)
       else if(!(strncmp(argv[i], "-poolhandle=" ,12))) {
          poolHandle = (char*)&argv[i][12];
       }
+      else if(!(strncmp(argv[i], "-environment=" ,13))) {
+         EnvironmentName = (const char*)&argv[i][13];
+         if(EnvironmentName[0] == 0x00) {
+            EnvironmentName = NULL;
+         }
+      }
       else if(!(strncmp(argv[i], "-input=" ,7))) {
          InputName = (const char*)&argv[i][7];
       }
@@ -523,7 +603,7 @@ int main(int argc, char** argv)
       }
       else {
          fprintf(stderr, "ERROR: Bad argument %s\n", argv[i]);
-         fprintf(stderr, "Usage: %s [-input=Input Name] [-output=Output Name] {-poolhandle=Pool Handle} {-quiet} {-maxretry=Trials} {-retrydelay=milliseconds} {-transmittimeout=milliseconds} {-keepaliveinterval=milliseconds} {-keepalivetimeout=milliseconds}\n", argv[0]);
+         fprintf(stderr, "Usage: %s [-environment=Environment Name] [-input=Input Name] [-output=Output Name] {-poolhandle=Pool Handle} {-quiet} {-maxretry=Trials} {-retrydelay=milliseconds} {-transmittimeout=milliseconds} {-keepaliveinterval=milliseconds} {-keepalivetimeout=milliseconds}\n", argv[0]);
          exit(1);
       }
    }
@@ -535,14 +615,34 @@ int main(int argc, char** argv)
    info.ri_csp_identifier = CID_COMPOUND(CID_GROUP_POOLUSER, random64());
 #endif
 
+   if(EnvironmentName) {
+      EnvironmentSize = sha1_computeHashOfFile(EnvironmentName, (uint8_t*)&EnvironmentHash);
+      if(EnvironmentSize == 0) {
+         fprintf(stderr, "ERROR: Unable to compute hash of environment file %s\n",
+                 EnvironmentName);
+         exit(1);
+      }
+   }
+
 
    if(!Quiet) {
-      puts("Scripting Pool User - Version 1.0");
+      puts("Scripting Pool User - Version 2.0");
       puts("=================================\n");
       printf("Pool Handle         = %s\n", poolHandle);
+      printf("Environment Name    = ");
+      if(EnvironmentName) {
+         printf("%s (", EnvironmentName);
+         for(i = 0; i < (int)sizeof(EnvironmentHash); i++) {
+            printf("%02x", (unsigned int)EnvironmentHash[i]);
+         }
+         printf(", %llu KiB)\n", EnvironmentSize / 1024);
+      }
+      else {
+         puts("(none)");
+      }
       printf("Input Name          = %s\n", InputName);
       printf("Output Name         = %s\n\n", OutputName);
-      printf("Max Retry           = %u [ms]\n", MaxRetry);
+      printf("Max Retry           = %u\n", MaxRetry);
       printf("Retry Delay         = %llu [ms]\n", RetryDelay / 1000);
       printf("Transmit Timeout    = %llu [ms]\n", TransmitTimeout / 1000);
       printf("Keep-Alive Interval = %llu [ms]\n", KeepAliveInterval / 1000);
@@ -596,7 +696,7 @@ int main(int argc, char** argv)
          if(received > 0) {
 
             /* ====== Notification ================================= */
-            if(flags & MSG_RSERPOOL_NOTIFICATION) {             
+            if(flags & MSG_RSERPOOL_NOTIFICATION) {
                notification = (union rserpool_notification*)&buffer;
                if(HasAssignedPE == true) {   /* Skip printing if no PE has accepted the request yet. */
                   newLogLine(stdout);
@@ -624,7 +724,7 @@ int main(int argc, char** argv)
       if( (result == SSCR_OKAY) && (nextTimer <= getMicroTime()) ) {
          /* ====== KeepAlive transmission ========================== */
          if(!KeepAliveTransmitted) {
-            result = sendKeepAlive(sd);
+            result = sendKeepAliveMessage(sd);
          }
 
          /* ====== KeepAlive timeout =============================== */
@@ -647,6 +747,7 @@ int main(int argc, char** argv)
             newLogLine(stdout);
             puts("FAILOVER ...");
             fflush(stdout);
+            HasAssignedPE = false;   /* All work is lost -> start from scratch ... */
          }
          InfoString[0] = 0x00;
          nextTimer = getMicroTime() + randomizeWaitingTime(RetryDelay);
@@ -670,5 +771,5 @@ int main(int argc, char** argv)
    rsp_close(sd);
    rsp_cleanup();
    rsp_freeinfo(&info);
-   return((result == SSCR_COMPLETE) ? 0 : 1);
+   return(((result == SSCR_COMPLETE) && (ExitStatus == 0)) ? 0 : 1);
 }
