@@ -1,4 +1,4 @@
-/* $Id: scriptingpackets.h 2240 2009-10-21 08:11:57Z dreibh $
+/* $Id: scriptingpackets.h 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,11 @@
 #define SPT_KEEPALIVE          0x04
 #define SPT_KEEPALIVE_ACK      0x05
 #define SPT_STATUS             0x06
+#define SPT_ENVIRONMENT        0x07
 
 #define SD_MAX_DATASIZE        1400
 #define SR_MAX_INFOSIZE          64
+#define SE_HASH_SIZE             20   /* 160-bit SHA-1 hash */
 
 struct ScriptingCommonHeader
 {
@@ -61,10 +63,9 @@ struct NotReady
    char                         Info[];
 } __attribute__((packed));
 
-#define SSNR_FULLY_LOADED     0x0000001
+#define SSNR_FULLY_LOADED     0x00000001
 #define SSNR_OUT_OF_RESOURCES 0x00000002
 
-
 struct Ready
 {
    struct ScriptingCommonHeader Header;
@@ -100,4 +101,12 @@ struct Status
    uint32_t                     Status;
 } __attribute__((packed));
 
+struct Environment
+{
+   struct ScriptingCommonHeader Header;
+   uint8_t                      Hash[SE_HASH_SIZE];
+} __attribute__((packed));
+
+#define SEF_UPLOAD_NEEDED   (1 << 0)
+
 #endif
