@@ -1,4 +1,4 @@
-/* $Id: scriptingservice.cc 2260 2009-10-25 14:57:10Z dreibh $
+/* $Id: scriptingservice.cc 2480 2011-02-01 15:16:49Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,16 +35,22 @@
 
 #include <unistd.h>
 #include <stdlib.h>
+#include <math.h>
 #include <errno.h>
 #include <assert.h>
 #include <signal.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
+#include <sys/wait.h>
 
 
-#define INPUT_NAME  "input.data"
-#define OUTPUT_NAME "output.data"
-#define STATUS_NAME "status.txt"
+#define ENVIRONMENT_NAME  "environment.data"
+#define INPUT_NAME        "input.data"
+#define OUTPUT_NAME       "output.data"
+#define STATUS_NAME       "status.txt"
+
+
+EnvironmentCache ScriptingServer::Cache;
 
 
 // ###### Constructor #######################################################
@@ -60,6 +66,15 @@ ScriptingServer::ScriptingServer(
    Directory[0]           = 0x00;
    LastKeepAliveTimeStamp = 0;
    WaitingForKeepAliveAck = false;
+   WaitingForEnvironment  = false;
+   NeedsEnvironment       = false;
+   GotEnvironment         = false;
+
+   UploadSize             = 0;
+   DownloadSize           = 0;
+   UploadStarted          = 0;
+   ProcessingStarted      = 0;
+   DownloadStarted        = 0;
 }
 
 
@@ -83,17 +98,57 @@ bool ScriptingServer::start()
 
 
 // ###### Create a Scripting thread #########################################
-TCPLikeServer* ScriptingServer::scriptingServerFactory(int sd, void* userData, const uint32_t peIdentifier)
+TCPLikeServer* ScriptingServer::scriptingServerFactory(
+                  int sd, void* userData, const uint32_t peIdentifier)
 {
    return(new ScriptingServer(sd, (ScriptingServerSettings*)userData));
 }
 
 
+// ###### Initialize service ################################################
+bool ScriptingServer::initializeService(void* userData)
+{
+   const ScriptingServerSettings* settings = (const ScriptingServerSettings*)userData;
+   if(Cache.initializeCache(settings->CacheDirectory.c_str(),
+                            settings->CacheMaxSize, settings->CacheMaxEntries,
+                            stdlog) == false) {
+      fputs("ERROR: Unable to initialize environment cache!\n", stderr);
+      return(false);
+   }
+   return(true);
+}
+
+
+// ###### Finish service ####################################################
+void ScriptingServer::finishService(void* userData)
+{
+   // const ScriptingServerSettings* settings = (const ScriptingServerSettings*)userData;
+   Cache.cleanUp();
+}
+
+
+// ###### Print parameters ##################################################
+void ScriptingServer::scriptingPrintParameters(const void* userData)
+{
+   const ScriptingServerSettings* settings = (const ScriptingServerSettings*)userData;
+
+   puts("Scripting Parameters:");
+   printf("   Keep Temp Dirs          = %s\n", (settings->KeepTempDirs == true) ? "yes" : "no");
+   printf("   Verbose Mode            = %s\n", (settings->VerboseMode == true) ? "yes" : "no");
+   printf("   Transmit Timeout        = %u [ms]\n", settings->TransmitTimeout);
+   printf("   Keep-Alive Interval     = %u [ms]\n", settings->KeepAliveInterval);
+   printf("   Keep-Alive Timeout      = %u [ms]\n", settings->KeepAliveTimeout);
+   printf("   Cache Max Size          = %llu [KiB]\n", settings->CacheMaxSize / 1024);
+   printf("   Cache Max Entries       = %u\n", settings->CacheMaxEntries);
+   printf("   Cache Directory         = %s\n", settings->CacheDirectory.c_str());
+}
+
+
 // ###### Reject new session when server is fully loaded ####################
 void ScriptingServer::rejectNewSession(int sd)
 {
-   char    buffer[sizeof(NotReady) + SR_MAX_INFOSIZE];
-   utsname systemInfo;
+   char           buffer[sizeof(NotReady) + SR_MAX_INFOSIZE];
+   struct utsname systemInfo;
 
    if(uname(&systemInfo) != 0) {
       systemInfo.nodename[0] = 0x00;
@@ -116,9 +171,23 @@ void ScriptingServer::rejectNewSession(int sd)
 // ###### Initialize session ################################################
 EventHandlingResult ScriptingServer::initializeSession()
 {
-   char buffer[sizeof(Ready) + SR_MAX_INFOSIZE];
-   utsname systemInfo;
+   // ====== Create directory and get file names ============================
+   safestrcpy((char*)&Directory, "/tmp/rspSS-XXXXXX", sizeof(Directory));
+   if(mkdtemp((char*)&Directory) == NULL) {
+      printTimeStamp(stdlog);
+      fprintf(stdlog, "S%04d: Unable to generate temporary directory!\n",
+            RSerPoolSocketDescriptor);
+      return(EHR_Abort);
+   }
+   snprintf((char*)&EnvironmentName,  sizeof(EnvironmentName), "%s/%s",  Directory, ENVIRONMENT_NAME);
+   snprintf((char*)&InputName,  sizeof(InputName), "%s/%s",  Directory, INPUT_NAME);
+   snprintf((char*)&OutputName, sizeof(OutputName), "%s/%s", Directory, OUTPUT_NAME);
+   snprintf((char*)&StatusName, sizeof(StatusName), "%s/%s", Directory, STATUS_NAME);
 
+
+   // ====== Send Ready message =============================================
+   char           buffer[sizeof(Ready) + SR_MAX_INFOSIZE];
+   struct utsname systemInfo;
    if(uname(&systemInfo) != 0) {
       systemInfo.nodename[0] = 0x00;
    }
@@ -133,7 +202,7 @@ EventHandlingResult ScriptingServer::initializeSession()
 
    setSyncTimer(getMicroTime() + (1000ULL * Settings.TransmitTimeout));   // Timeout for first upload message
 
-   
+
    const ssize_t sent  = rsp_sendmsg(RSerPoolSocketDescriptor,
                                      (const char*)ready, readyLength, 0,
                                      0, htonl(PPID_SP), 0, 0, 0, Settings.TransmitTimeout);
@@ -156,7 +225,7 @@ void ScriptingServer::finishSession(EventHandlingResult result)
       int  success;
 
       snprintf((char*)&sscmd, sizeof(sscmd), "scriptingcontrol cleanup %s %d %s",
-               Directory, ChildProcess,
+               Directory, (int)ChildProcess,
                (Settings.KeepTempDirs == true) ? "keeptempdirs" : "");
       snprintf((char*)&callcmd, sizeof(callcmd), "if [ -e ./scriptingcontrol ] ; then ./%s ; else %s ; fi", sscmd, sscmd);
 
@@ -176,54 +245,40 @@ void ScriptingServer::finishSession(EventHandlingResult result)
 }
 
 
-// ###### Print parameters ##################################################
-void ScriptingServer::scriptingPrintParameters(const void* userData)
-{
-   const ScriptingServerSettings* settings = (const ScriptingServerSettings*)userData;
-
-   puts("Scripting Parameters:");
-   printf("   Keep Temp Dirs          = %s\n", (settings->KeepTempDirs == true) ? "yes" : "no");
-   printf("   Verbose Mode            = %s\n", (settings->VerboseMode == true) ? "yes" : "no");
-   printf("   Transmit Timeout        = %u [ms]\n", settings->TransmitTimeout);
-   printf("   Keep-Alive Interval     = %u [ms]\n", settings->KeepAliveInterval);
-   printf("   Keep-Alive Timeout      = %u [ms]\n", settings->KeepAliveTimeout);
-}
-
-
 // ###### Handle Upload message #############################################
-EventHandlingResult ScriptingServer::handleUploadMessage(const char* buffer,
-                                                         size_t      bufferSize)
+EventHandlingResult ScriptingServer::handleUploadMessage(const Upload* upload)
+
 {
-   // ====== Create temporary directory =====================================
+   // ====== Create input file ==============================================
    if(UploadFile == NULL) {
-      // ====== Create directory and get file names =========================
-      safestrcpy((char*)&Directory, "/tmp/rspSS-XXXXXX", sizeof(Directory));
-      if(mkdtemp((char*)&Directory) == NULL) {
-         printTimeStamp(stdlog);
-         fprintf(stdlog, "S%04d: Unable to generate temporary directory!\n",
-                 RSerPoolSocketDescriptor);
-         return(EHR_Abort);
+      if(WaitingForEnvironment == true) {
+         sha1_init(&EnvironmentHashContext);
+         UploadFile = fopen(EnvironmentName, "w");
       }
-      snprintf((char*)&InputName, sizeof(InputName), "%s/%s", Directory, INPUT_NAME);
-      snprintf((char*)&OutputName, sizeof(OutputName), "%s/%s", Directory, OUTPUT_NAME);
-      snprintf((char*)&StatusName, sizeof(StatusName), "%s/%s", Directory, STATUS_NAME);
+      else {
+         UploadFile = fopen(InputName, "w");
+      }
 
-      // ====== Create input file ===========================================
-      UploadFile = fopen(InputName, "w");
       if(UploadFile == NULL) {
          printTimeStamp(stdlog);
-         fprintf(stdlog, "S%04d: Unable to create input file in directory \"%s\": %s!\n",
-                 RSerPoolSocketDescriptor, Directory, strerror(errno));
+         fprintf(stdlog, "S%04d: Unable to create input file \"%s\" in directory \"%s\": %s!\n",
+                 RSerPoolSocketDescriptor,
+                 (WaitingForEnvironment == true) ? EnvironmentName : InputName,
+                 Directory, strerror(errno));
          return(EHR_Abort);
       }
       printTimeStamp(stdlog);
-      fprintf(stdlog, "S%04d: Starting upload into directory \"%s\" ...\n",
-              RSerPoolSocketDescriptor, Directory);
+      fprintf(stdlog, "S%04d: Starting upload of %s into directory \"%s\" ...\n",
+              RSerPoolSocketDescriptor,
+              (WaitingForEnvironment == true) ? "environment" : "work package",
+              Directory);
+      UploadSize    = 0;
+      UploadStarted = getMicroTime();
    }
 
-   // ====== Write to input file ============================================
-   const Upload* upload = (const Upload*)buffer;
-   const size_t length  = bufferSize - sizeof(ScriptingCommonHeader);
+
+   // ====== Write data to input file =======================================
+   const size_t length = ntohs(upload->Header.Length) - sizeof(upload->Header);
    if(length > 0) {
       if(Settings.VerboseMode) {
          fputs(".", stdlog);
@@ -235,26 +290,121 @@ EventHandlingResult ScriptingServer::handleUploadMessage(const char* buffer,
                  RSerPoolSocketDescriptor, Directory, strerror(errno));
          return(EHR_Abort);
       }
+      if(WaitingForEnvironment == true) {
+         sha1_update(&EnvironmentHashContext, (uint8_t*)&upload->Data, length);
+      }
+      UploadSize += (unsigned long long)length;
       setSyncTimer(getMicroTime() + (1000ULL * Settings.TransmitTimeout));   // Timeout for next upload message
       return(EHR_Okay);
    }
+
+
+   // ====== Upload finished ================================================
    else {
       if(Settings.VerboseMode) {
          fputs("\n", stdlog);
       }
-      fclose(UploadFile);
-      fprintf(stdlog, "S%04d: Starting work in directory \"%s\" ...\n",
-              RSerPoolSocketDescriptor, Directory);
+      if(fclose(UploadFile) != 0) {
+         fprintf(stdlog, "S%04d: Close error for input file in directory \"%s\": %s!\n",
+                 RSerPoolSocketDescriptor, Directory, strerror(errno));
+         return(EHR_Abort);
+      }
       UploadFile = NULL;
-      EventHandlingResult result = sendStatus(0);
-      if(result == 0) {
-         return(startWorking());
+
+      ProcessingStarted = getMicroTime();
+
+      printTimeStamp(stdlog);
+      fprintf(stdlog, "S%04d: Upload completed (%s: %1.0lf KiB in %1.1lf s; %1.1lf KiB/s) => starting work in directory \"%s\" ...\n",
+              RSerPoolSocketDescriptor,
+              (WaitingForEnvironment == true) ? EnvironmentName : InputName,
+              ceil((double)UploadSize / 1024.0),
+              (ProcessingStarted - UploadStarted) / 1000000.0,
+              (double)((double)UploadSize / 1024.0) / ((ProcessingStarted - UploadStarted) / 1000000.0),
+              Directory);
+
+
+      // ====== Continue upload with environment file =======================
+      if((NeedsEnvironment == true) && (WaitingForEnvironment == false)) {
+         WaitingForEnvironment = true;   // Now, get the environment
+         return(EHR_Okay);
+      }
+
+      // ====== Upload of work package and environment completed ============
+      else {
+         if(WaitingForEnvironment == true) {
+            GotEnvironment = true;
+            sha1_final(&EnvironmentHashContext, (uint8_t*)&EnvironmentHash);
+            printTimeStamp(stdlog);
+            fprintf(stdlog, "S%04d: Got environment ",
+                    RSerPoolSocketDescriptor);
+            for(size_t i = 0; i < sizeof(EnvironmentHash); i++) {
+               fprintf(stdlog, "%02x", (unsigned int)EnvironmentHash[i]);
+            }
+            fputs("\n", stdlog);
+            Cache.storeInCache((const uint8_t*)&EnvironmentHash,
+                               EnvironmentName,
+                               UploadSize);
+         }
+
+         // ====== Start to do actual work ==================================
+         EventHandlingResult result = sendStatus(0);
+         if(result == 0) {
+            return(startWorking());
+         }
+         return(result);
       }
-      return(result);
    }
 }
 
 
+// ###### Handle Upload message #############################################
+EventHandlingResult ScriptingServer::handleEnvironmentMessage(
+                                        const Environment* environmentQuery)
+{
+   // ====== Check Environment query message ================================
+   if(ntohs(environmentQuery->Header.Length) < sizeof(Environment)) {
+      printTimeStamp(stdlog);
+      fprintf(stdlog, "S%04d: Received invalid Environment message\n",
+              RSerPoolSocketDescriptor);
+      return(EHR_Abort);
+   }
+
+   printTimeStamp(stdlog);
+   fprintf(stdlog, "S%04d: User requests environment ", RSerPoolSocketDescriptor);
+   for(size_t i = 0; i < sizeof(environmentQuery->Hash); i++) {
+      fprintf(stdlog, "%02x", (unsigned int)environmentQuery->Hash[i]);
+   }
+   fputs("\n", stdlog);
+
+
+   // ====== Look for environment in cache ==================================
+   if(Cache.copyFromCache((const uint8_t*)&environmentQuery->Hash,
+                          EnvironmentName) == false) {
+      NeedsEnvironment = true;
+   }
+   else {
+      GotEnvironment = true;
+   }
+
+
+   // ====== Tell user the result ===========================================
+   Environment environmentResponse;
+   environmentResponse.Header.Type   = SPT_ENVIRONMENT;
+   environmentResponse.Header.Flags  = (NeedsEnvironment == true) ? SEF_UPLOAD_NEEDED : 0;
+   environmentResponse.Header.Length = htons(sizeof(environmentResponse));
+   memcpy(&environmentResponse.Hash, &environmentQuery->Hash, sizeof(environmentResponse.Hash));
+   ssize_t sent = rsp_sendmsg(RSerPoolSocketDescriptor,
+                     (const char*)&environmentResponse, sizeof(environmentResponse), 0,
+                     0, htonl(PPID_SP), 0, 0, 0, Settings.TransmitTimeout);
+   if(sent != (ssize_t)sizeof(environmentResponse)) {
+      printTimeStamp(stdlog);
+      fprintf(stdlog, "S%04d: Environment transmission error: %s\n",
+               RSerPoolSocketDescriptor, strerror(errno));
+      return(EHR_Abort);
+   }
+   return(EHR_Okay);
+}
+
 // ###### Send status report ################################################
 EventHandlingResult ScriptingServer::sendStatus(const int exitStatus)
 {
@@ -285,9 +435,12 @@ EventHandlingResult ScriptingServer::performDownload()
 
    FILE* fh = fopen(OutputName, "r");
    if(fh != NULL) {
+      DownloadStarted = getMicroTime();
       printTimeStamp(stdlog);
-      fprintf(stdlog, "S%04d: Starting download of results in directory \"%s\" ...\n",
-              RSerPoolSocketDescriptor, Directory);
+      fprintf(stdlog, "S%04d: Processing completed (%1.1lf s) => starting download of results in directory \"%s\" ...\n",
+              RSerPoolSocketDescriptor,
+              (DownloadStarted - ProcessingStarted) / 1000000.0,
+              Directory);
       for(;;) {
          dataLength = fread((char*)&download.Data, 1, sizeof(download.Data), fh);
          if(dataLength >= 0) {
@@ -308,8 +461,17 @@ EventHandlingResult ScriptingServer::performDownload()
                        RSerPoolSocketDescriptor, strerror(errno));
                return(EHR_Abort);
             }
+            DownloadSize += (unsigned long long)dataLength;
          }
          if(dataLength <= 0) {
+            const unsigned long long downloadFinished = getMicroTime();
+            printTimeStamp(stdlog);
+            fprintf(stdlog, "S%04d: Download completed (%1.0lf KiB in %1.1lf s; %1.1lf KiB/s) => finished work in directory \"%s\".\n",
+                  RSerPoolSocketDescriptor,
+                  ceil((double)DownloadSize / 1024.0),
+                  (downloadFinished - DownloadStarted) / 1000000.0,
+                  (double)((double)DownloadSize / 1024.0) / ((downloadFinished - DownloadStarted) / 1000000.0),
+                  Directory);
             break;
          }
       }
@@ -332,7 +494,7 @@ EventHandlingResult ScriptingServer::performDownload()
 EventHandlingResult ScriptingServer::startWorking()
 {
    assert(ChildProcess == 0);
-     
+
    printTimeStamp(stdlog);
    fprintf(stdlog, "S%04d: Starting work in directory \"%s\" ...\n",
            RSerPoolSocketDescriptor, Directory);
@@ -342,15 +504,19 @@ EventHandlingResult ScriptingServer::startWorking()
       const int stdlogFD = fileno(stdlog);
       dup2(stdlogFD, STDOUT_FILENO);
       dup2(stdlogFD, STDERR_FILENO);
-    
+
       // ====== Run script ==================================================
       execlp("./scriptingcontrol",
              "scriptingcontrol",
-             "run", Directory, INPUT_NAME, OUTPUT_NAME, STATUS_NAME, (char*)NULL);
+             "run", Directory, INPUT_NAME, OUTPUT_NAME, STATUS_NAME,
+             (GotEnvironment == true) ? ENVIRONMENT_NAME : NULL,
+             (char*)NULL);
       // Try standard location ...
       execlp("scriptingcontrol",
              "scriptingcontrol",
-             "run", Directory, INPUT_NAME, OUTPUT_NAME, STATUS_NAME, (char*)NULL);
+             "run", Directory, INPUT_NAME, OUTPUT_NAME, STATUS_NAME,
+             (GotEnvironment == true) ? ENVIRONMENT_NAME : NULL,
+             (char*)NULL);
       perror("Failed to start scriptingcontrol");
       exit(1);
    }
@@ -384,7 +550,7 @@ bool ScriptingServer::hasFinishedWork(int& exitStatus)
 EventHandlingResult ScriptingServer::syncTimerEvent(const unsigned long long now)
 {
    EventHandlingResult result = EHR_Okay;
-   
+
    switch(State) {
      case SS_Upload:
         printTimeStamp(stdlog);
@@ -392,7 +558,7 @@ EventHandlingResult ScriptingServer::syncTimerEvent(const unsigned long long now
                 RSerPoolSocketDescriptor, Directory);
         result = EHR_Abort;
       break;
-      
+
      case SS_Working:
         // ====== Check whether work is complete ============================
         int                 exitStatus;
@@ -402,7 +568,7 @@ EventHandlingResult ScriptingServer::syncTimerEvent(const unsigned long long now
               result = performDownload();
            }
         }
-        
+
         // ====== Keep-Alive handling =======================================
         else {
            // ------ Send KeepAlive -----------------------------------------
@@ -424,7 +590,7 @@ EventHandlingResult ScriptingServer::syncTimerEvent(const unsigned long long now
            }
         }
       break;
-      
+
      default:
       break;
    }
@@ -486,12 +652,14 @@ EventHandlingResult ScriptingServer::handleMessage(const char* buffer,
                                                    uint16_t    streamID)
 {
    /* ====== Check message header ======================================== */
+/*
    if(ntohl(ppid) != PPID_SP) {
       printTimeStamp(stdlog);
       fprintf(stdlog, "S%04d: Received message has wrong PPID $%08x!\n",
               RSerPoolSocketDescriptor, ntohl(ppid));
       return(EHR_Abort);
    }
+*/
    if(bufferSize < sizeof(struct ScriptingCommonHeader)) {
       printTimeStamp(stdlog);
       fprintf(stdlog, "S%04d: Received message of %u bytes does not even contain header!\n",
@@ -499,7 +667,7 @@ EventHandlingResult ScriptingServer::handleMessage(const char* buffer,
       return(EHR_Abort);
    }
    const ScriptingCommonHeader* header = (const ScriptingCommonHeader*)buffer;
-   if(ntohs(header->Length) != bufferSize) {
+   if(bufferSize < ntohs(header->Length)) {
       printTimeStamp(stdlog);
       fprintf(stdlog, "S%04d: Received message has %u bytes but %u bytes are expected!\n",
               RSerPoolSocketDescriptor,
@@ -511,10 +679,13 @@ EventHandlingResult ScriptingServer::handleMessage(const char* buffer,
    switch(State) {
       case SS_Upload:
          if(header->Type == SPT_UPLOAD) {
-            return(handleUploadMessage(buffer, bufferSize));
+            return(handleUploadMessage((const Upload*)buffer));
+         }
+         else if(header->Type == SPT_ENVIRONMENT) {
+            return(handleEnvironmentMessage((const Environment*)buffer));
          }
        break;
-      case SS_Working:       
+      case SS_Working:
          if(header->Type == SPT_KEEPALIVE) {
             return(handleKeepAliveMessage());
          }
@@ -526,7 +697,7 @@ EventHandlingResult ScriptingServer::handleMessage(const char* buffer,
           // All messages here are unexpected!
        break;
    }
-   
+
    // ====== Print unexpected message =======================================
    printTimeStamp(stdlog);
    fprintf(stdlog, "S%04d: Received unexpected message $%02x in state #%u!\n",
