@@ -1,4 +1,4 @@
-/* $Id: server.cc 2260 2009-10-25 14:57:10Z dreibh $
+/* $Id: server.cc 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -539,8 +539,11 @@ start:
       settings.KeepTempDirs      = false;
       settings.VerboseMode       = false;
       settings.TransmitTimeout   = 30000;
-      settings.KeepAliveInterval = 10000;
+      settings.KeepAliveInterval = 15000;
       settings.KeepAliveTimeout  = 10000;
+      settings.CacheMaxSize      = 128*1024*1024;
+      settings.CacheMaxEntries   = 16;
+      settings.CacheDirectory    = "";
       for(int i = 1;i < argc;i++) {
          if(!(strncmp(argv[i], "-ssmaxthreads=", 14))) {
             maxThreads = atol((const char*)&argv[i][14]);
@@ -563,20 +566,37 @@ start:
          else if(!(strncmp(argv[i], "-sskeepalivetimeout=", 20))) {
             settings.KeepAliveTimeout = atol((const char*)&argv[i][20]);
          }
+         else if(!(strncmp(argv[i], "-sscachemaxsize=", 16))) {
+            settings.CacheMaxSize = 1024ULL * atoll((const char*)&argv[i][16]);
+            if(settings.CacheMaxSize < 128*1024) {
+               settings.CacheMaxSize = 128*1024;
+            }
+         }
+         else if(!(strncmp(argv[i], "-sscachemaxentries=", 19))) {
+            settings.CacheMaxEntries = atol((const char*)&argv[i][19]);
+            if(settings.CacheMaxEntries < 1) {
+               settings.CacheMaxEntries = 1;
+            }
+         }
+         else if(!(strncmp(argv[i], "-sscachedirectory=", 18))) {
+            settings.CacheDirectory = (const char*)&argv[i][18];
+         }
       }
       if(!policyChanged) {
          loadInfo.rli_load_degradation = (unsigned int)rint((1.0 / maxThreads) * (double)PPV_MAX_LOAD_DEGRADATION);
          loadInfo.rli_policy = PPT_LEASTUSED_DEGRADATION;
       }
 
-      TCPLikeServer::poolElement("Scripting Server - Version 1.0",
+      TCPLikeServer::poolElement("Scripting Server - Version 2.0",
                                  (poolHandle != NULL) ? poolHandle : "ScriptingPool",
                                  &info, &loadInfo,
                                  maxThreads,
                                  ScriptingServer::scriptingServerFactory,
                                  ScriptingServer::scriptingPrintParameters,
                                  ScriptingServer::rejectNewSession,
-                                 NULL, NULL, NULL,
+                                 ScriptingServer::initializeService,
+                                 ScriptingServer::finishService,
+                                 NULL,
                                  (void*)&settings,
                                  localAddressSet, localAddresses,
                                  reregInterval, runtimeLimit, quiet, (daemonPIDFile != NULL),
