@@ -1,4 +1,4 @@
-/* $Id: standardservices.cc 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: standardservices.cc 2409 2011-01-01 20:38:28Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -142,11 +142,20 @@ void DaytimeServer::handleNotification(const union rserpool_notification* notifi
                          notification->rn_session_change.rsc_session,
                          0x00000000, 0,
                          0, 0, 0);
+#ifdef SOLARIS
+      rsp_sendmsg(RSerPoolSocketDescriptor,
+                  NULL, 0,
+                  MSG_EOF,
+                  notification->rn_session_change.rsc_session,
+                  0x00000000, 0,
+                  0, 0, 0);
+#else
       rsp_sendmsg(RSerPoolSocketDescriptor,
                   NULL, 0, 0,
                   notification->rn_session_change.rsc_session,
                   0x00000000, 0,
                   0, SCTP_EOF, 0);
+#endif
    }
 }
 
