@@ -1,4 +1,4 @@
-/* $Id: tcplikeserver.cc 2258 2009-10-25 14:54:45Z dreibh $
+/* $Id: tcplikeserver.cc 2520 2011-02-19 10:05:52Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2009 by Thomas Dreibholz
+ * Copyright (C) 2002-2011 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ double TCPLikeServer::getLoad() const
 void TCPLikeServer::setLoad(double load)
 {
    ServerList->lock();
-   
+
    CHECK(ServerList != NULL);
    CHECK(ServerList->LoadSum >= Load);
    if((load < 0.0) || (load > 1.0)) {
@@ -132,7 +132,10 @@ void TCPLikeServer::setLoad(double load)
    ServerList->unlock();
 
    if(oldTotalLoad != newTotalLoad) {
-      ext_write(ServerList->SystemNotificationPipe, "!", 1);
+      const ssize_t result = ext_write(ServerList->SystemNotificationPipe, "!", 1);
+      if(result <= 0) {
+         perror("Writing to system notification pipe failed");
+      }
    }
 }
 
@@ -277,11 +280,19 @@ void TCPLikeServer::run()
    finishSession(eventHandlingResult);
    if((eventHandlingResult == EHR_Abort) ||
       (eventHandlingResult == EHR_Shutdown)) {
+#ifdef SOLARIS
+      rsp_sendmsg(RSerPoolSocketDescriptor,
+                  NULL, 0,
+                  (eventHandlingResult == EHR_Abort) ? MSG_ABORT : MSG_EOF,
+                  0, 0, 0, 0, 0,
+                  0);
+#else
       rsp_sendmsg(RSerPoolSocketDescriptor,
                   NULL, 0, 0,
                   0, 0, 0, 0,
                   (eventHandlingResult == EHR_Abort) ? SCTP_ABORT : SCTP_EOF,
                   0);
+#endif
    }
    lock();
    Finished = true;
@@ -307,7 +318,7 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                                 unsigned int         reregInterval,
                                 unsigned int         runtimeLimit,
                                 const bool           quiet,
-                                const bool           daemonMode, 
+                                const bool           daemonMode,
                                 struct TagItem*      tags)
 {
    if(rsp_initialize(info) < 0) {
@@ -351,7 +362,8 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                      // ====== Print program title ================================
                      if(!quiet) {
                         puts(programTitle);
-                        for(size_t i = 0;i < strlen(programTitle);i++) {
+                        const size_t programTitleSize = strlen(programTitle);
+                        for(size_t i = 0;i < programTitleSize;i++) {
                            printf("=");
                         }
                         const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
@@ -429,7 +441,7 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                            if(serverSet.handleRemovalsAndTimers() > 0) {
                               int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
                               if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
-                                 logerror("Unable to update backlog using rsp_listen()");                                 
+                                 logerror("Unable to update backlog using rsp_listen()");
                               }
                            }
 
@@ -476,9 +488,11 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                                            rejectNewSession(newRSerPoolSocket);
                                         }
                                         rsp_close(newRSerPoolSocket);
-                                        printTimeStamp(stdlog);
-                                        fputs("Rejected new session, since server is fully loaded\n", stdlog);
-                                        fflush(stdlog);
+                                        if(!quiet) {
+                                           printTimeStamp(stdlog);
+                                           fputs("Rejected new session, since server is fully loaded\n", stdlog);
+                                           fflush(stdlog);
+                                        }
                                      }
                                   }
                               }
@@ -504,7 +518,10 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                               // ====== Clear notification pipe ===================
                               if(pollfds[1].revents & POLLIN) {
                                  char buffer[1024];
-                                 ext_read(systemNotificationPipe[0], (char*)&buffer, sizeof(buffer));
+                                 const ssize_t result = ext_read(systemNotificationPipe[0], (char*)&buffer, sizeof(buffer));
+                                 if(result < 0) {
+                                    perror("Reading from system notification pipe failed");
+                                 }
                               }
 
                            }
